using FitConnect.Interfaces;
using Newtonsoft.Json;

public record MockSettings(List<string> PrivateDecryptionKeys, string PrivateSigningKey,
    string PublicKeyEncryption, string PublicKeySignatureVerification, string SenderClientId,
    string SenderClientSecret, string SubscriberClientId, string SubscriberClientSecret,
    Guid DestinationId, string ServiceIdentifier, string CallbackSecret,
    IEnumerable<Guid> DestinationList, Uri JsonSchemaUri, Uri XmlSchemaUri) : IFitConnectSettings {
}

public record ConfigFile(
    ClientConfig Sender, ClientConfig Subscriber, string ServiceIdentifier,
    List<DestinationConfig> Destinations);

public record DestinationConfig(Guid Id, List<string> PrivateDecryptionKey,
    string PrivateSigningKey, string PublicEncryptionKey, string PublicSignatureVerificationKey,
    string Note, string Name,
    Uri JsonSchemaUri, Uri XmlSchemaUri);

public record ClientConfig(string ClientId, string ClientSecret, string CallbackSecret);


public static class ConfigurationLoader {
    public static IFitConnectSettings? GetSettingsFromFiles(string name) {
        try {
            var credentials =
                JsonConvert.DeserializeObject<ConfigFile>(
                    File.ReadAllText("./credentials.json"))!;

            var destinationSettings = credentials.Destinations.First(d => d.Name == name);

            var senderClientId = credentials.Sender.ClientId;
            var senderClientSecret = credentials.Sender.ClientSecret;
            var subscriberClientId = credentials.Subscriber.ClientId;
            var subscriberClientSecret = credentials.Subscriber.ClientSecret;
            var serviceIdentifier = credentials.ServiceIdentifier;
            var callbackSecret = credentials.Subscriber.CallbackSecret;

            var destinationId = destinationSettings.Id.ToString();


            return new MockSettings(
                destinationSettings.PrivateDecryptionKey,
                destinationSettings.PrivateSigningKey,
                destinationSettings.PublicEncryptionKey,
                destinationSettings.PublicEncryptionKey,
                senderClientId,
                senderClientSecret,
                subscriberClientId,
                subscriberClientSecret,
                Guid.Parse(destinationId),
                serviceIdentifier, callbackSecret,
                credentials.Destinations.Select(d => d.Id),
                destinationSettings.JsonSchemaUri, destinationSettings.XmlSchemaUri);
        }
        catch {
            Console.WriteLine("Files not found, using environment variables");
            return null;
        }
    }
}
